<?php

namespace Drupal\iconset\Plugin\Iconset;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Plugin\DerivativeInspectionInterface;
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\iconset\IconsetInterface;
use Drupal\iconset\Asset\AssetInterface;
use Drupal\iconset\Plugin\IconHandlerFileInterface;
use Drupal\iconset\Plugin\IconHandlerFileBase;

/**
 * Handler that finds and manages icons in directories.
 *
 * The functionality defined in this class, is used to find asset files in a
 * directory, and has an internal plugin handler to handle each of the files
 * found matching the file extention (in the plugin derived plugin definition).
 *
 * This allows this class to implement directory searching for icons of
 * different types. Icon handlers that can be use as internal DirectoryHandler
 * plugins are defined by "hook_iconset_icon_directory_handler_type()".
 *
 * Essentially plugins of this class are wrappers for an underlying single file
 * icon handler plugin.
 *
 * @IconsetIconHandler(
 *   id = "directory",
 *   label = @Translation("Icon directory"),
 *   help = @Translation("Find and discover icons from directories."),
 *   deriver = "Drupal\iconset\Plugin\Deriver\IconDirectoryDeriver"
 * )
 */
class DirectoryHandler extends IconHandlerFileBase implements DerivativeInspectionInterface, ContainerFactoryPluginInterface {

  /**
   * Creates a new instance of the DirectoryHandler icon plugin.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The derived plugin definition.
   * @param \Drupal\Core\File\FileSystemInterface $filesystem
   *   The filesystem service.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $icon_handler_manager
   *   The plugin manager for icon handler plugins.
   *
   * @see \Drupal\iconset\Plugin\Deriver\IconDirectoryDeriver
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, FileSystemInterface $filesystem, PluginManagerInterface $icon_handler_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $filesystem);

    // Create the internal icon plugin handler for the individual icon files
    // found in this directory matching the file pattern.
    $this->iconHandler = $icon_handler_manager->createInstance($this->getDerivativeId());

    if (!$this->iconHandler instanceof IconHandlerFileInterface) {
      throw new \InvalidArgumentException('boomed');
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_system'),
      $container->get('plugin.manager.iconset.icon_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFileExtensions() {
    return $this->handler->getFileExtensions();
  }

  /**
   * {@inheritdoc}
   */
  public function createAssets($asset_info, IconsetInterface $iconset) {
    try {
      $extensions = array_map('preg_quote', $this->iconHandler->getFileExtensions(), ['/']);
      $extPattern = '/\.(' . implode('|', $extensions) . ')$/';

      $directory = rtrim($this->getFilepath($asset_info, $iconset), '/');
      $files = $this->filesystem->scanDirectory($directory, $extPattern, [
        'key' => 'basename',
      ]);

      $assets = [];
      foreach ($files as $file) {
        $asset = $this->iconHandler->createAssets("/{$file->uri}", $iconset);

        if (is_array($asset)) {
          $assets = array_merge($assets, $asset);
        }
        elseif ($asset) {
          $assets[] = $asset;
        }
      }

      return $assets;
    }
    catch (\Exception $e) {
      $this
        ->getLogger('iconset')
        ->error('Unable to directory for SVG files at @line of @file: @message', [
          '@line' => $e->getLine(),
          '@file' => $e->getFile(),
          '@message' => $e->getMessage(),
        ]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getJsSettings() {
    return $this->iconHandler->getJsSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formatJson(AssetInterface $asset) {
    return $this->iconHandler->formatJson($asset);
  }

  /**
   * {@inheritdoc}
   */
  public function build($icon_id, AssetInterface $asset, array $options = []) {
    return $this->iconHandler->build($icon_id, $asset, $options);
  }

}
